/*
 * ipmi_con.c
 *
 * Interface code for handling IPMI connections
 *
 * Copyright (c) 2003 by FORCE Computers.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  This
 * file and program are licensed under a BSD style license.  See
 * the Copying file included with the OpenHPI distribution for
 * full licensing terms.
 *
 * Authors:
 *     Thomas Kanngieser <thomas.kanngieser@fci.com>
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <errno.h>


__BEGIN_DECLS
#include <openhpi.h>
__END_DECLS


#include "ipmi_con.h"
#include "ipmi_domain.h"


static void IpmiLogDataMsg( const cIpmiAddr &addr, const cIpmiMsg &msg );


cIpmiCon::cIpmiCon()
  : m_domain( 0 ), m_is_open( false )
{
}


cIpmiCon::~cIpmiCon()
{
}


int
cIpmiCon::IfOpen()
{
  return 0;
}


void
cIpmiCon::IfClose()
{
}


void
cIpmiCon::IfEvent()
{
}


int
cIpmiCon::Open()
{
  if ( IsOpen() )
       return 0;

  int rv = IfOpen();
  
  if ( rv == 0 )
       m_is_open = true;

  return rv;
}


void
cIpmiCon::Close()
{
  if ( !IsOpen() )
       return;

  IfClose();

  m_is_open = false;
}


int
cIpmiCon::Cmd( const cIpmiAddr &addr, const cIpmiMsg &msg,
               cIpmiAddr &rsp_addr, cIpmiMsg &rsp_msg )
{
  int rv;
   
  assert( msg.m_data_len <= dIpmiMaxMsgLength );

  IpmiLogData( "cmd " );
  IpmiLogDataMsg( addr, msg );
  IpmiLogData( "\n" );

  rv = IfCmd( addr, msg, rsp_addr, rsp_msg );

  if ( !rv )
     {
       IpmiLogData( "rsp " );
       IpmiLogDataMsg( rsp_addr, rsp_msg );
       IpmiLogData( "\n" );
     }
  else if ( rv == ETIMEDOUT )
       IpmiLogData( "tim\n" );
  else 
       IpmiLogData( "err\n" );

  return rv;
}


int
cIpmiCon::ExecuteCmd( const cIpmiAddr &addr, const cIpmiMsg &msg, cIpmiMsg &rsp_msg )
{
  cIpmiAddr rsp_addr;

  return Cmd( addr, msg, rsp_addr, rsp_msg );
}


void
cIpmiCon::Event()
{
  IfEvent();
}


void
cIpmiCon::HandleEvent( const cIpmiAddr &addr, const cIpmiMsg &msg )
{
  IpmiLogData( "evt " );
  IpmiLogDataMsg( addr, msg );
  IpmiLogData( "\n" );

  assert( m_domain );
  m_domain->AddAsyncEvent( addr, msg );
}


static void
IpmiLogDataMsg( const cIpmiAddr &addr, const cIpmiMsg &msg )
{
  char str[1024];
  char *s = str;

  // addr
  switch( addr.m_type )
     {
       case eIpmiAddrTypeIpmb:
            s += sprintf( s, "%02x %02x %02x",
                          eIpmiAddrTypeSystemInterface, addr.m_channel, addr.m_lun );
            break;

       case eIpmiAddrTypeSystemInterface:
            s += sprintf( s, "%02x %02x %02x %02x",
                          eIpmiAddrTypeSystemInterface, addr.m_channel, addr.m_lun, addr.m_slave_addr );
            break;

       case eIpmiAddrTypeIpmbBroadcast:
            s += sprintf( s, "%02x %02x %02x %02x",
                          eIpmiAddrTypeIpmbBroadcast, addr.m_channel, addr.m_lun, addr.m_slave_addr );
     }

  // msg
  s += sprintf( s, "  %02x %02x %02x ",
		msg.m_netfn, msg.m_cmd, msg.m_data_len );

  const unsigned char *p = msg.m_data;

  for( int i = 0; i < msg.m_data_len; i++ )
       s += sprintf( s, " %02x", *p++ );

  IpmiLogData( str );
}
