/*
 * ipmi_con.h
 *
 * abstract interface for handling IPMI connections
 *
 * Copyright (c) 2003 by FORCE Computers.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  This
 * file and program are licensed under a BSD style license.  See
 * the Copying file included with the OpenHPI distribution for
 * full licensing terms.
 *
 * Authors:
 *     Thomas Kanngieser <thomas.kanngieser@fci.com>
 */

#ifndef dIpmiCon_h
#define dIpmiCon_h


#ifndef dIpmiLog_h
#include "ipmi_log.h"
#endif

#ifndef dIpmiAddr_h
#include "ipmi_addr.h"
#endif

#ifndef dIpmiMsg_h
#include "ipmi_msg.h"
#endif

#ifndef dIpmiCmd_h
#include "ipmi_cmd.h"
#endif


#define dIpmiUsernameMax   16

class cIpmiDomain;

class cIpmiCon
{
protected:
  cIpmiDomain *m_domain;
  bool         m_is_open;

public:
  cIpmiCon();
  virtual ~cIpmiCon();

  void SetDomain( cIpmiDomain *domain )
  {
    m_domain = domain;
  }

  bool IsOpen() { return m_is_open; }

  virtual int  IfOpen();
  virtual void IfClose();
  virtual int  IfCmd( const cIpmiAddr &addr, const cIpmiMsg &msg,
                      cIpmiAddr &rsp_addr, cIpmiMsg &rsp_msg ) = 0;
  virtual void IfEvent();

  int   Open();
  void  Close();
  int   Cmd( const cIpmiAddr &addr, const cIpmiMsg &msg,
             cIpmiAddr &rsp_addr, cIpmiMsg &rsp_msg );
  
  int   ExecuteCmd( const cIpmiAddr &addr, const cIpmiMsg &msg,
                    cIpmiMsg &rsp_msg );
  void  Event();

  virtual void HandleEvent( const cIpmiAddr &addr, const cIpmiMsg &msg );
};


#endif
