/*      -*- linux-c -*-
 *
 *      $Id: snmp_bc_sel.c,v 1.12 2003/12/10 21:21:53 djudkovi Exp $
 *
 * (C) Copyright IBM Corp. 2003
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  This
 * file and program are licensed under a BSD style license.  See
 * the Copying file included with the OpenHPI distribution for
 * full licensing terms.
 *
 * Authors:
 *      Sean Dague <http://dague.net/sean>
 *
 */

#include <SaHpi.h>
#include <snmp_bc_sel.h>
#include <snmp_util.h>
#include <snmp_bc.h>
#include <time.h>

static int get_bc_sel_size(struct snmp_session *ss)
{
        struct snmp_value run_value;
        char oid[50];
        int i = 1;
        /*
          yes there clearly must be a much less dumb way of doing this.
          wait for Konrad next week to figure this out 
        */
        do {
                sprintf(oid, "%s.%d", BC_SEL_INDEX_OID, i);
                i++;
        } while(snmp_get(ss,oid,&run_value) == 0);
        
        /* think about it, and it makes sense */
        i -= 2;
        return i;
}

int snmp_bc_get_sel_info(void *hnd, SaHpiResourceIdT id, SaHpiSelInfoT *info) 
{
        struct snmp_value first_value;
        struct oh_handler_state *handle = hnd;
        struct snmp_bc_hnd *custom_handle = (struct snmp_bc_hnd *)handle->data;
        bc_sel_entry sel_entry;
        struct tm curtime;
        char oid[50];
        int i = 1;
        
        SaHpiSelInfoT sel = {
                .Size = 512, /* this is clearly a guess but looks about right 
                              * from the 75% full errors I've seen */
                .Enabled = SAHPI_TRUE,
                .OverflowFlag = SAHPI_FALSE,
                .OverflowAction = SAHPI_SEL_OVERFLOW_DROP,
                .DeleteEntrySupported = SAHPI_FALSE
        };
        
        sprintf(oid, "%s.%d", BC_SEL_ENTRY_OID, i);
        /* we need first value to figure out what our update time is */
        snmp_get(custom_handle->ss,oid,&first_value);
        
        if(first_value.type == ASN_OCTET_STR) {
                if(snmp_bc_parse_sel_entry(first_value.string, &sel_entry) < 0) {
                        dbg("Couldn't get first date");
                } else {
                        sel.UpdateTimestamp = 
                                (SaHpiTimeT) mktime(&sel_entry.time) * 1000000000;
                }
        }
        
        if(get_bc_sp_time(custom_handle->ss,&curtime) == 0) {
                sel.CurrentTime = 
                        (SaHpiTimeT) mktime(&curtime) * 1000000000;
        }
        
        sel.Entries = get_bc_sel_size(custom_handle->ss);
        *info = sel;
        
        return 0;
}

/**
 * snmp_bc_get_sel_entry:
 * @hnd: 
 * @id: 
 * @current: 
 * @prev: 
 * @next: 
 * @entry: 
 * 
 * See saHpiEventLogEntryGet for params
 * 
 * Return value: 0 on success, < 0 on error
 **/
int snmp_bc_get_sel_entry(void *hnd, SaHpiResourceIdT id, SaHpiSelEntryIdT current,
                          SaHpiSelEntryIdT *prev, SaHpiSelEntryIdT *next,
                          SaHpiSelEntryT *entry)
{
        struct snmp_value get_value;
        struct oh_handler_state *handle = hnd;
        struct snmp_bc_hnd *custom_handle = handle->data;
        bc_sel_entry sel_entry;
        SaHpiSelEntryT tmpentry;
        char oid[50];
        int oidinst = 1;
        
        if(current == SAHPI_OLDEST_ENTRY) {
                oidinst = get_bc_sel_size(custom_handle->ss);
                *next = oidinst - 1;
                *prev = SAHPI_NO_MORE_ENTRIES;
        } else if(current == SAHPI_NEWEST_ENTRY) {
                oidinst = 1;
                *next = SAHPI_NO_MORE_ENTRIES;
                *prev = oidinst + 1;
        } else {
                int final = get_bc_sel_size(custom_handle->ss);
                if(current > final) {
                        dbg("Gone past the end of the log");
                        return -1;
                }
                oidinst = current;
                if(current < final) {
                        *prev = current + 1;
                } else {
                        *prev = SAHPI_NO_MORE_ENTRIES;
                }
                
                if(current > 1) {
                        *next = current - 1;
                } else {
                        *next = SAHPI_NO_MORE_ENTRIES;
                }
        }

        sprintf(oid, "%s.%d", BC_SEL_ENTRY_OID, oidinst);
        /* we need first value to figure out what our update time is */
        snmp_get(custom_handle->ss,oid,&get_value);
        
        if(get_value.type == ASN_OCTET_STR) {
                if(snmp_bc_parse_sel_entry(get_value.string, &sel_entry) < 0) {
                        dbg("Couldn't parse SEL Entry");
                        return -1;
                }
        } else {
                dbg("Couldn't fetch SEL Entry");
                return -1;
        }

        tmpentry.EntryId = current;
        tmpentry.Timestamp = (SaHpiTimeT) mktime(&sel_entry.time) * 1000000000;
        tmpentry.Event.Severity = sel_entry.sev;
        tmpentry.Event.Timestamp = (SaHpiTimeT) mktime(&sel_entry.time) * 1000000000;
        /* TODO: do this for real, sel_entry.source will give this sort of */
        tmpentry.Event.Source = id;
        tmpentry.Event.EventType = SAHPI_ET_OEM;
        tmpentry.Event.EventDataUnion.OemEvent.MId = 2;
        /* TODO: break down events better.  It would be nice if there was
           a TEXT type event for informational messages */
        strncpy(tmpentry.Event.EventDataUnion.OemEvent.OemEventData,
                sel_entry.text, SAHPI_OEM_EVENT_DATA_SIZE - 1);
        tmpentry.Event.EventDataUnion.OemEvent.OemEventData
                [SAHPI_OEM_EVENT_DATA_SIZE - 1] = '\0';
        *entry = tmpentry;

        return 0;
}


/**
 * snmp_bc_set_sel_time:
 * @hnd: 
 * @id: 
 * @time: 
 * 
 * 
 * 
 * Return value: 
 **/
int snmp_bc_set_sel_time(void *hnd, SaHpiResourceIdT id, SaHpiTimeT time)
{
        struct oh_handler_state *handle = hnd;
        struct snmp_bc_hnd *custom_handle = (struct snmp_bc_hnd *)handle->data;
        struct tm tv;
        time_t tt;
        tt = time / 1000000000;
        
        localtime_r(&tt, &tv);
        
        set_bc_sp_time(custom_handle->ss,&tv);
                
        return SA_ERR_HPI_UNSUPPORTED_API;
}

/**
 * snmp_bc_add_sel_entry:
 * @hnd: 
 * @id: 
 * @Event: 
 * 
 * Add is not supported with tihs hardware, so -1 is always returned
 * 
 * Return value: -1
 **/
int snmp_bc_add_sel_entry(void *hnd, SaHpiResourceIdT id, const SaHpiSelEntryT *Event)
{
        return SA_ERR_HPI_INVALID_CMD;
}

/**
 * snmp_bc_del_sel_entry:
 * @hnd: 
 * @id: 
 * @sid: 
 * 
 * Delete is not supported with this hardware, so -1 is always returned
 * 
 * Return value: -1
 **/
int snmp_bc_del_sel_entry(void *hnd, SaHpiResourceIdT id, SaHpiSelEntryIdT sid)
{
        return SA_ERR_HPI_INVALID_CMD;
}


/**
 * snmp_bc_parse_sel_entry:
 * @text: text as returned by snmpget call for an event log entry
 * @sel: blade center system event log
 * 
 * This call is used to create a blade center sel entry from the returned
 * snmp string.  Another transform will have to happen to turn this into 
 * an SAHPI sel entry. 
 * 
 * Return value: 0 for success, -1 for format error, -2 for premature data termination
 **/
int snmp_bc_parse_sel_entry(char * text, bc_sel_entry * sel) 
{
        bc_sel_entry ent;
        char level[8];
        char * start = text;
        
        /* Severity first */
        if(sscanf(start,"Severity:%7s",level)) {
                if(strcmp(level,"INFO") == 0) {
                        ent.sev = SAHPI_INFORMATIONAL;
                } else if(strcmp(level,"WARN") == 0) {
                        ent.sev = SAHPI_MINOR;
                } else if(strcmp(level,"ERR") == 0) {
                        ent.sev = SAHPI_CRITICAL;
                } else {
                        ent.sev = SAHPI_DEBUG;
                }
        } else {
                dbg("Couldn't parse Severity from Blade Center Log Entry");
                return -1;
        }
                
        while(start && (strncmp(start,"Source:",7) != 0)) { start++; }
        if(!start) { return -2; }

        if(!sscanf(start,"Source:%19s",ent.source)) {
                dbg("Couldn't parse Source from Blade Center Log Entry");
                return -1;
        }

        while(start && (strncmp(start,"Name:",5) != 0)) { start++; }
        if(!start) { return -2; }

        if(!sscanf(start,"Name:%19s",ent.sname)) {
                dbg("Couldn't parse Name from Blade Center Log Entry");
                return -1;
        }
        
        while(start && (strncmp(start,"Date:",5) != 0)) { start++; }
        if(!start) { return -2; }
        
        if(sscanf(start,"Date:%2d/%2d/%2d  Time:%2d:%2d:%2d",
                  &ent.time.tm_mon, &ent.time.tm_mday, &ent.time.tm_year, 
                  &ent.time.tm_hour, &ent.time.tm_min, &ent.time.tm_sec)) {
                /* TODO: real test for DST */
                ent.time.tm_isdst = 0;
                /* that word, I do not think it means what you think it means */
                ent.time.tm_mon--;
                ent.time.tm_year += 100;
        } else {
                dbg("Couldn't parse Date/Time from Blade Center Log Entry");
                return -1;
        }
        
        while(start && (strncmp(start,"Text:",5) != 0)) { start++; }
        if(!start) { return -2; }
        
        /* advance to data */
        start += 5;
        strncpy(ent.text,start,BC_SEL_ENTRY_STRING - 1);
        ent.text[BC_SEL_ENTRY_STRING - 1] = '\0';
        
        *sel = ent;
        return 0;
}

int get_bc_sp_time(struct snmp_session *ss, struct tm *time) {
        struct snmp_value get_value;
        struct tm tmptime;

        snmp_get(ss,BC_DATETIME_OID,&get_value);
        if(get_value.type == ASN_OCTET_STR) {
                if(sscanf(get_value.string,"%2d/%2d/%4d,%2d:%2d:%2d",
                          &tmptime.tm_mon, &tmptime.tm_mday, &tmptime.tm_year, 
                          &tmptime.tm_hour, &tmptime.tm_min, &tmptime.tm_sec)) {
                        /* TODO: real test for DST */
                        tmptime.tm_isdst = 0;
                        /* that word, I do not think it means what you think it means */
                        tmptime.tm_mon--;
                        tmptime.tm_year -= 1900;
                } else {
                        dbg("Couldn't parse Date/Time from Blade Center SP");
                        return -1;
                }
        } else {
                dbg("Couldn't fetch Blade Center SP Date/Time Entry");
                return -1;
        }
        *time = tmptime;
        return 0;
}

int set_bc_sp_time(struct snmp_session *ss, struct tm *time) {
        // struct snmp_value set_value;
        // char timestr[20];
        /* yes, this doesn't do anything right now */
        printf("%2d/%2d/%4d,%2d:%2d:%2d\n",time->tm_mon + 1, time->tm_mday,
               time->tm_year + 1900, time->tm_hour, time->tm_min, time->tm_sec);
        
        return -1;
}
